/*
 * (c) Copyright 1993, 1994, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * US Government Users Restricted Rights
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
#include "unitsquare.h"

#include <GL/gl.h>

static float vertices[][3] = {
  {-1.0, -1.0, 0.0},
  {0.0, -1.0, 0.0},
  {0.0, 0.0, 0.0},
  {-1.0, 0.0, 0.0},

  {0.0, -1.0, 0.0},
  {1.0, -1.0, 0.0},
  {1.0, 0.0, 0.0},
  {0.0, 0.0, 0.0},

  {-1.0, 0.0, 0.0},
  {0.0, 0.0, 0.0},
  {0.0, 1.0, 0.0},
  {-1.0, 1.0, 0.0},

  {0.0, 0.0, 0.0},
  {1.0, 0.0, 0.0},
  {1.0, 1.0, 0.0},
  {0.0, 1.0, 0.0}

};

static float texcoord[][2] = {
  {0.0, 0.0},
  {0.5, 0.0},
  {0.5, 0.5},
  {0.0, 0.5},

  {0.5, 0.0},
  {1.0, 0.0},
  {1.0, 0.5},
  {0.5, 0.5},

  {0.0, 0.5},
  {0.5, 0.5},
  {0.5, 1.0},
  {0.0, 1.0},

  {0.5, 0.5},
  {1.0, 0.5},
  {1.0, 1.0},
  {0.5, 1.0}
};

unitsquare::unitsquare()
{
}

void unitsquare::open()
{
}

void unitsquare::draw()
{
  glBegin(GL_QUADS);

  glTexCoord2fv(texcoord[0]);
  glVertex3fv(vertices[0]);
  glTexCoord2fv(texcoord[1]);
  glVertex3fv(vertices[1]);
  glTexCoord2fv(texcoord[2]);
  glVertex3fv(vertices[2]);
  glTexCoord2fv(texcoord[3]);
  glVertex3fv(vertices[3]); 

  glTexCoord2fv(texcoord[4]);
  glVertex3fv(vertices[4]);
  glTexCoord2fv(texcoord[5]);
  glVertex3fv(vertices[5]);
  glTexCoord2fv(texcoord[6]);
  glVertex3fv(vertices[6]);
  glTexCoord2fv(texcoord[7]);
  glVertex3fv(vertices[7]); 

  glTexCoord2fv(texcoord[8]);
  glVertex3fv(vertices[8]);
  glTexCoord2fv(texcoord[9]);
  glVertex3fv(vertices[9]);
  glTexCoord2fv(texcoord[10]);
  glVertex3fv(vertices[10]);
  glTexCoord2fv(texcoord[11]);
  glVertex3fv(vertices[11]); 

  glTexCoord2fv(texcoord[12]);
  glVertex3fv(vertices[12]);
  glTexCoord2fv(texcoord[13]);
  glVertex3fv(vertices[13]);
  glTexCoord2fv(texcoord[14]);
  glVertex3fv(vertices[14]);
  glTexCoord2fv(texcoord[15]);
  glVertex3fv(vertices[15]); 

  glEnd();
}

void unitsquare::draw_backwards()
{
  glBegin(GL_QUADS);

  glTexCoord2fv(texcoord[3]);
  glVertex3fv(vertices[3]);
  glTexCoord2fv(texcoord[2]);
  glVertex3fv(vertices[2]);
  glTexCoord2fv(texcoord[1]);
  glVertex3fv(vertices[1]);
  glTexCoord2fv(texcoord[0]);
  glVertex3fv(vertices[0]); 

  glTexCoord2fv(texcoord[7]);
  glVertex3fv(vertices[7]);
  glTexCoord2fv(texcoord[6]);
  glVertex3fv(vertices[6]);
  glTexCoord2fv(texcoord[5]);
  glVertex3fv(vertices[5]);
  glTexCoord2fv(texcoord[4]);
  glVertex3fv(vertices[4]); 

  glTexCoord2fv(texcoord[11]);
  glVertex3fv(vertices[11]);
  glTexCoord2fv(texcoord[10]);
  glVertex3fv(vertices[10]);
  glTexCoord2fv(texcoord[9]);
  glVertex3fv(vertices[9]);
  glTexCoord2fv(texcoord[8]);
  glVertex3fv(vertices[8]); 

  glTexCoord2fv(texcoord[15]);
  glVertex3fv(vertices[15]);
  glTexCoord2fv(texcoord[14]);
  glVertex3fv(vertices[14]);
  glTexCoord2fv(texcoord[13]);
  glVertex3fv(vertices[13]);
  glTexCoord2fv(texcoord[12]);
  glVertex3fv(vertices[12]); 

  glEnd();
}
